/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2007 The LS-Universe Team

  Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

/***************************************************************************
****************************************************************************

  Original source -> soso_pub <-

  Link: http://www.codeproject.com/string/yasr.asp

  Modified by Andymon

****************************************************************************
****************************************************************************/

#ifndef CSTR_H
#define CSTR_H

#include <windows.h>
#include <tchar.h>

class CStr
{
public:
	// all methods marked with throw() will throw std::bad_alloc
	// if memory allocation is needed and it fails

	// every allocation (including the first one) adds an amount
	// of CSTR_EXTRA_SIZE TCHARs to internal buffer	so the buffer
	// will not be resized every time a small operation is performed;
	static int		CSTR_EXTRA_SIZE;
	
	// construction
					CStr() throw();
					~CStr();
					CStr( const CStr &str ) throw();
					CStr( LPCTSTR pszStr ) throw();
					CStr( int allocSize ) throw();
	CStr&			operator = ( const CStr& str ) throw();
	CStr&			operator = ( LPCTSTR pszStr ) throw();
	CStr&			operator = ( int iValue ) throw();
	CStr&			operator = ( long lValue ) throw();
	CStr&			operator = ( unsigned long ulValue ) throw();
	CStr&			operator = ( double dValue ) throw();
			
	// internal buffer access
	TCHAR*			Buffer() const;
	TCHAR*			Buffer();
	int				BufferSize() const;
	void			Realloc( int size ) throw();
	void			Compact() throw();

	// attributes
	int				Length() const;
	BOOL			IsEmpty() const;

	// comparision
	BOOL			operator == ( LPCTSTR pszStr ) const;
	BOOL			operator != ( const LPCTSTR& str ) const;
	int				Compare( LPCTSTR pszStr ) const;
	int				CompareNoCase( LPCTSTR pszStr ) const;

	// accessors
					operator LPCTSTR () const;
					operator int () const;
					operator long () const;
					operator unsigned long () const;
					operator double () const;

	TCHAR&			operator [] ( int pos );
	const TCHAR&	operator [] ( int pos ) const;
	
	CStr			Left( int size ) const throw();
	CStr			Right( int size ) const throw();
	CStr			Mid( int start, int size ) const throw();
	
					// number of occurences
	int				FindCount( LPCTSTR pszStr ) const; 
					// same as above only starting at startPos
	int				FindCount( int startPos, LPCTSTR pszStr ) const; 
					// zero-based index of the 1st occurence
	int				Find( LPCTSTR pszStr ) const;
					// zero-based index of the nth occurence
	int				FindNth( int nth, LPCTSTR pszStr ) const;
					// zero-based index of the 1st occurence starting
					// from the end of the string
	int				ReverseFind( LPCTSTR pszStr ) const;
					// zero-based index of the nth occurence starting
					// from the end of the string
	int				ReverseFindNth( int nth, LPCTSTR pszStr ) const;

	// operations
					// sets string to  ""
	CStr&			Empty();
					// replaces all string characters with chr
	CStr&			Fill( const TCHAR chr );
	CStr&			Trim();
	CStr&			TrimLeft();
	CStr&			TrimRight();
	CStr&			Lower();
	CStr&			Upper();
					// insert pszStr at pos
	CStr&			Insert( int pos, LPCTSTR pszStr ) throw();
	CStr&			Prepend( LPCTSTR pszStr ) throw();
	CStr&			Append( LPCTSTR pszStr ) throw();
					// remove len characters starting from pos
	CStr&			Remove( int pos, int len ) throw();
					// truncates the string from pos
	CStr&			Trunc( int pos ) ;
					// replace all occurences of pszOld with pszNew
	CStr&			Replace( LPCTSTR pszOld, LPCTSTR pszNew ) throw();
					// same as above only starting from startPos
	CStr&			Replace( int startPos, LPCTSTR pszOld, LPCTSTR pszNew ) throw();
	CStr&			Format( LPCTSTR pszFormat, ... ) throw();
	CStr&			FormatV( LPCTSTR pszFormat, va_list args ) throw();
	
	// numerics ( get/set )
	int				GetInt() const;
	long			GetLong() const;
	unsigned long	GetULong() const;
	double			GetDouble() const;
	CStr&			SetInt( const int iValue ) throw();
	CStr&			SetLong( const long lValue ) throw();
	CStr&			SetULong( const unsigned long ulValue ) throw();
	CStr&			SetDouble( const double dValue ) throw();

	// << append operator
	friend CStr&	operator << ( CStr &str, LPCTSTR pszStr ) throw();
	friend CStr&	operator << ( CStr &str, int iValue ) throw();
	friend CStr&	operator << ( CStr &str, long lValue ) throw();
	friend CStr&	operator << ( CStr &str, unsigned long ulValue ) throw();
	friend CStr&	operator << ( CStr &str, double dValue ) throw();

private:
	// the following constants are used internally; do not modify
	static int		FORMAT_EXTRA_SIZE;
	static int		INT_SIZE;
	static int		LONG_SIZE;
	static int		ULONG_SIZE;

	// internal methods
	void			Alloc_( int size ) throw();
	void			Free_();
	void			Init_( int allocSize = 1 ) throw();
	int				PaddedSize_( int size );
	void			UpdateBufferSize_( const int size ) throw();
	void			StrCopy_( LPCTSTR source, int size );
	int				TLEN( int len );

	TCHAR			*m_pBuf;
	int				m_size;
	int				m_len;
};

#endif